# Decorator to add some special features to a User
# TODO: This is a strong relationship to DeviseAuthentication and will to stop work if we use anything different!! (se)
DeviseAuthentication::User.class_eval do
    if Rails.env != "test"
        # Add jwt support to the user model
        include Devise::JWT::RevocationStrategies::JTIMatcher

        devise :jwt_authenticatable, jwt_revocation_strategy: self
        field :jti, :type => String, :unique => true, :index => true
        before_validation :initialize_jti_once
    end

    before_destroy  :account_about_to_delete

    # Add current organisation
    def jwt_payload
        init_current_membership
        {
            'membership_id' => @current_membership.id,
            'jti' => jti
        }
    end

    # Required by devise-jwt in order to find the correct user
    def self.primary_key
        "id"
    end

    # Called by devise-swt in order to find the user..
    def self.find_by(params)
        self.find(params["id"])
    end

    def self.revoke_jwt(_payload, user)
        return if user.nil?
        user.update(:jti => generate_jti)
    end

    def initialize_jti_once
        initialize_jti if self.jti.nil?
    end


    # TODO: Broadcast to UserChannel to notify the UI about it.
    def account_about_to_delete
        # Delete all documents originally imported by this user.
        Nukleus::Document.where(origin_user_id: self.id).each do |document|
            document.destroy
        end

        # Move all assigned documents back to its origin or delete it if this user is the origin
        Nukleus::Document.where(user_id: self.id).each do |document|
            if document.origin_user_id.blank?  # It is assigned to myself => Returning makes no sense
                document.destroy
            else
                document.back_to_origin(self, I18n.t("document_retuned_due_to_deletion"))
            end
        end

    end

end
